@@ -52,6 +52,7 @@ static char *help_pages[] = {
 
 static char *state_title[] = {
 	N_("Welcome to Bug Buddy"),
+	N_("Collecting debug info"),
 	N_("Select a Product or Application"),
 	N_("Select a Component"),
 	N_("Frequently Reported Bugs"),
@@ -64,6 +65,67 @@ static char *state_title[] = {
 
 #define d(x)
 
+static void
+mail_write_template (void)
+{
+	GtkTextBuffer *desc_buffer;
+	GtkTextIter iter;
+
+	switch (druid_data.bug_type) {
+	case BUG_CRASH:
+		buddy_set_text ("desc-text", TEXT_BUG_CRASH);
+		break;
+	case BUG_DEBUG:
+		buddy_set_text ("desc-text", TEXT_BUG_CRASH);
+		break;
+	case BUG_WRONG:
+		buddy_set_text ("desc-text", TEXT_BUG_WRONG);
+		break;
+	case BUG_I18N:
+		buddy_set_text ("desc-text", TEXT_BUG_I18N);
+		break;
+	case BUG_DOC:
+		buddy_set_text ("desc-text", TEXT_BUG_DOC);
+		break;
+	case BUG_REQUEST:
+		buddy_set_text ("desc-text", TEXT_BUG_REQUEST);
+		break;
+	}
+
+	/* set the cursor at the beginning of the second line
+	* in the description page */
+	desc_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW
+						(GET_WIDGET ("desc-text")));
+	gtk_text_buffer_get_iter_at_line (desc_buffer, &iter, 1);
+	gtk_text_buffer_place_cursor (desc_buffer, &iter);
+                                                                                
+}
+
+
+static void
+mail_lock_text (void)
+{
+	GtkTextView *view;
+	GtkTextBuffer *buffer;
+	GtkTextIter start, end;
+	GtkTextTagTable *tag_table;
+	GtkTextTag *tag;
+
+	view = GTK_TEXT_VIEW (GET_WIDGET ("email-text"));
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+
+	gtk_text_buffer_get_start_iter (buffer, &start);
+
+	if ( gtk_text_iter_forward_search (&start, "Description:",
+					   GTK_TEXT_SEARCH_TEXT_ONLY,
+					   NULL, &end, NULL)) {
+				  
+		tag_table = gtk_text_buffer_get_tag_table (buffer);
+		tag = gtk_text_tag_table_lookup (tag_table, "lock_tag");
+		gtk_text_buffer_apply_tag (buffer, tag, &start, &end);
+	}
+}
+
 void
 druid_set_sensitive (gboolean prev, gboolean next, gboolean cancel)
 {
@@ -82,8 +144,10 @@ void
 on_druid_about_clicked (GtkWidget *button, gpointer data)
 {
 	static GtkWidget *about;
+	GdkPixbuf *pixbuf = NULL;
 	static const char *authors[] = {
 		"Jacob Berkman  <jacob@bug-buddy.org>",
+		"Fernando Herrera  <fherrera@onirica.com>",
 		NULL
 	};
 
@@ -99,14 +163,20 @@ on_druid_about_clicked (GtkWidget *button, gpointer data)
 		return;
 	}
 		
+	pixbuf = gdk_pixbuf_new_from_file(BUDDY_ICONDIR "/bug-buddy.png", NULL);
+	
 	about = gnome_about_new (_("Bug Buddy"), VERSION,
 				 "Copyright (C) 1999, 2000, 2001, 2002, Jacob Berkman\n"
-				 "Copyright 2000, 2001, 2002 Ximian, Inc.",
+				 "Copyright 2000, 2001, 2002 Ximian, Inc.\n"
+				 "Copyright 2003 Fernando Herrera",
 				 _("The graphical bug reporting tool for GNOME."),
 				 authors,
 				 documentors,
-				 NULL, NULL);
+				 NULL, pixbuf);
 
+	if (pixbuf != NULL)
+		gdk_pixbuf_unref (pixbuf);
+	
 	g_signal_connect (G_OBJECT (about), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
 			  &about);
@@ -126,6 +196,16 @@ on_druid_about_clicked (GtkWidget *button, gpointer data)
 	gtk_widget_show (about);
 }
 
+
+void invalid_description (void) { 
+	GtkWidget *w;
+	
+       	w = gtk_message_dialog_new (GTK_WINDOW (get_widget (("druid-window"),(G_STRLOC))), 0, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, error_string);
+       	gtk_dialog_set_default_response (GTK_DIALOG (w), GTK_RESPONSE_OK);
+       	gtk_dialog_run (GTK_DIALOG (w));
+       	gtk_widget_destroy (w);
+}
+
 void
 druid_set_state (BuddyState state)
 {
@@ -160,7 +240,15 @@ druid_set_state (BuddyState state)
 				       state);	
 
 	switch (druid_data.state) {
+	case STATE_INTRO:
+		break;
 	case STATE_GDB:
+		if (druid_data.bug_type==BUG_DEBUG) {
+			GtkWidget *nb;
+			nb = GET_WIDGET ("gdb-notebook");
+			gtk_notebook_set_current_page (GTK_NOTEBOOK (nb), 1);
+			gtk_widget_hide (GET_WIDGET ("debugging-options-button"));
+		}
 		break;
 	case STATE_PRODUCT:
 		if (druid_data.download_in_progress)
@@ -172,7 +260,7 @@ druid_set_state (BuddyState state)
 		/* nothing to do */
 		break;		
 	case STATE_DESC:
-		/* nothing to do */
+		mail_write_template();
 		break;
 	case STATE_EMAIL_CONFIG:
 		/* FIXME: change next icon */
@@ -185,6 +273,7 @@ druid_set_state (BuddyState state)
 		buddy_set_text ("email-text", s);
 		g_free (s);
 		on_email_group_toggled (NULL, NULL);
+		mail_lock_text ();
 		break;
 	case STATE_FINISHED:
 		/* print a summary yo */
@@ -219,7 +308,12 @@ on_druid_prev_clicked (GtkWidget *w, gpointer data)
 			newstate = STATE_PRODUCT;
 			break;
 		}
-		newstate = STATE_GDB;
+		if (druid_data.bug_type==BUG_CRASH ||
+		    druid_data.bug_type==BUG_DEBUG) {
+			newstate = STATE_GDB;
+			break;
+		}
+		newstate = STATE_INTRO;
 		break;
 	case STATE_MOSTFREQ:
 		if (!druid_data.component_skipped) {
@@ -230,14 +324,40 @@ on_druid_prev_clicked (GtkWidget *w, gpointer data)
 			newstate = STATE_PRODUCT;
 			break;
 		}
-		newstate = STATE_GDB;
+		if (druid_data.bug_type==BUG_CRASH ||
+		    druid_data.bug_type==BUG_DEBUG) {
+			newstate = STATE_GDB;
+			break;
+		}
+		newstate = STATE_INTRO;
 		break;
 	case STATE_COMPONENT:
 		if (!druid_data.product_skipped) {
 			newstate = STATE_PRODUCT;
 			break;
 		}
-		newstate = STATE_GDB;
+		if (druid_data.bug_type==BUG_CRASH ||
+		    druid_data.bug_type==BUG_DEBUG) {
+			newstate = STATE_GDB;
+			break;
+		}
+		newstate = STATE_INTRO;
+		break;
+	case STATE_PRODUCT:
+		if (druid_data.bug_type==BUG_CRASH ||
+		    druid_data.bug_type==BUG_DEBUG) {
+			newstate = STATE_GDB;
+			break;
+		}
+		newstate = STATE_INTRO;
+		break;
+
+	case STATE_GDB:
+		if (druid_data.bug_type==BUG_CRASH) {
+			newstate = STATE_GDB;
+			break;
+		}
+		newstate = STATE_INTRO;
 		break;
 	default:
 		break;
@@ -401,14 +521,49 @@ text_is_sensical (const gchar *text, int sensitivity)
 	   it is probably nonsenical.  Also require a space */
 	int chars[256] = { 0 };
 	guint uniq = 0;
+	guint count = 0;
 
 	if (!text || !*text)
 		return FALSE;
 	
-	for ( ; *text; text++)
+	for ( ; *text; text++) {
 		if (!chars[(guchar)*text])
 			chars[(guchar)*text] = ++uniq;
-	
+		if (count==0 && (int)*text==71) {
+			count=1;
+			continue;
+		}
+		if (count==5 && (int)*text==101) {
+			count=6;
+			continue;
+		}
+		if (count==2 && (int)*text==105) {
+			count=3;
+			continue;
+		}
+		if (count==4 && (int)*text==110) {
+			count=5;
+			continue;
+		}
+		if (count==1 && (int)*text==117) {
+			count=2;
+			continue;
+		}
+		if (count==6 && (int)*text==115) {
+			count=7;
+			continue;
+		}
+		if (count==3 && (int)*text==110) {
+			count=4;
+			continue;
+		}
+		if (count==7 && (int)*text==115) {
+			invalid_description();
+			return TRUE;
+		}
+		count=0;
+
+	}
 	d(g_message ("%d", uniq));
 
 	return chars[' '] && uniq >= sensitivity;
@@ -512,25 +667,6 @@ submit_ok (void)
 		RESPONSE_CANCEL
 	};
 
-	if (druid_data.submit_type != SUBMIT_FILE) {
-		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
-					    0,
-					    GTK_MESSAGE_QUESTION,
-					    GTK_BUTTONS_NONE,
-					    _("Submit this bug report now?"));
-		gtk_dialog_add_buttons (GTK_DIALOG (w),
-					GTK_STOCK_CANCEL, RESPONSE_CANCEL,
-					_("_Submit"), RESPONSE_SUBMIT,
-					NULL);
-		gtk_dialog_set_default_response (GTK_DIALOG (w),
-						 RESPONSE_SUBMIT);
-		if (RESPONSE_SUBMIT != gtk_dialog_run (GTK_DIALOG (w)))
-			goto submit_ok_out;
-
-		gtk_widget_destroy (w);
-		w = NULL;
-	}
-
 	buf = g_string_new (NULL);
 
 	name = buddy_get_text ("email-name-entry");
@@ -596,7 +732,7 @@ submit_ok (void)
 			goto submit_ok_out;
 		}
 
-		s = g_strdup_printf (_("Your bug report has been submitted to:\n\n        <%s>\n\nThanks!"), to);
+		s = g_strdup_printf (_("Your bug report has been submitted to:\n\n        <%s>\n\nBug reporting is an important part of making Free Software. Thank you for helping."), to);
 	}
 
 	buddy_set_text ("finished-label", s);
@@ -676,6 +812,48 @@ select_component_row (char *component_name)
         gtk_tree_view_columns_autosize (view);
 }
 	
+static void
+select_version (GSList *list, char *version)
+{
+
+	if (version && g_slist_find_custom (list, version ,(GCompareFunc)strcmp)!=NULL) {
+		druid_data.version = strdup (version);
+		gtk_option_menu_set_history (GTK_OPTION_MENU (GET_WIDGET ("the-version-menu")),
+					     g_slist_position (list, g_slist_find_custom (list, version, 
+											  (GCompareFunc)strcmp)));
+	} else {
+		/* Try to found a A.B.X version */
+		char *xversion;
+
+		xversion = strdup (version);
+		xversion[strlen (xversion)-1] = 'x';
+
+		if (g_slist_find_custom (list, xversion ,(GCompareFunc)strcmp)!=NULL) {
+			druid_data.version = strdup (xversion);
+			gtk_option_menu_set_history (GTK_OPTION_MENU (GET_WIDGET ("the-version-menu")),
+						     g_slist_position (list, g_slist_find_custom (list, xversion, 
+												  (GCompareFunc)strcmp)));
+		} else 
+			gtk_option_menu_set_history (GTK_OPTION_MENU (GET_WIDGET ("the-version-menu")),
+						     g_slist_position (list, g_slist_find_custom (list, "unspecified", 
+												  (GCompareFunc)strcmp)));
+		g_free (xversion);
+	}
+}
+
+static void
+select_severity (GSList *list, char *severity)
+{
+	if (severity && g_slist_find_custom (list, severity, (GCompareFunc)strcmp)!=NULL) {
+		druid_data.severity = strdup (severity);
+		gtk_option_menu_set_history (GTK_OPTION_MENU (GET_WIDGET ("severity-list")),
+					     g_slist_position (list, g_slist_find_custom (list, severity,
+											  (GCompareFunc)strcmp)));
+	}
+}
+
+
+
 
 void
 on_druid_next_clicked (GtkWidget *w, gpointer data)
@@ -687,13 +865,44 @@ on_druid_next_clicked (GtkWidget *w, gpointer data)
 	newstate = druid_data.state + 1;
 
 	switch (druid_data.state) {
+	case STATE_INTRO:
+	{
+		products_list_load ();
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("type-wrong-button")))) {
+			druid_data.bug_type = BUG_WRONG;
+			newstate = STATE_PRODUCT;
+			break;
+		}
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("type-doc-button")))) {
+			druid_data.bug_type = BUG_DOC;
+			newstate = STATE_PRODUCT;
+			break;
+		}
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("type-i18n-button")))) {
+			druid_data.bug_type = BUG_I18N;
+			newstate = STATE_PRODUCT;
+			break;
+		}
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("type-request-button")))) {
+			druid_data.bug_type = BUG_REQUEST;
+			newstate = STATE_PRODUCT;
+			break;
+		}
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("type-debug-button")))) {
+			druid_data.bug_type = BUG_DEBUG;
+			newstate = STATE_GDB;
+			buddy_set_text ("gdb_label", _("Please, select the core file or running application to debug."));
+			break;
+		}
+		newstate = STATE_PRODUCT;
+		break;
+	}
 	case STATE_GDB:
 	{
 		BugzillaApplication *application;
 		BugzillaProduct *product;
 		BugzillaBTS *bts;
 
-		products_list_load ();
 		if (!druid_data.current_appname)
 			break;
 		
@@ -717,18 +926,25 @@ on_druid_next_clicked (GtkWidget *w, gpointer data)
 		if (application->component) {
 			bugzilla_add_mostfreq (druid_data.product->bts);
 			druid_data.component = g_hash_table_lookup(product->components, application->component);
-			if (!druid_data.component)
+			if (!druid_data.component) {
+				update_version_menu (product);
+				select_version (product->versions, druid_data.version);
+				select_severity (druid_data.product->bts->severities, "critical");
 				break;
+			}
 				
 			newstate++;
 			druid_data.component_skipped = TRUE;
-			if (g_slist_find (druid_data.product->bts->severities, "critical")!=NULL); {
+			if (g_slist_find (druid_data.product->bts->severities, "critical")!=NULL)
 				druid_data.severity = "critical";
-			}
 			if (!druid_data.product->bts->bugs) {
 				newstate++;
 				druid_data.mostfreq_skipped = TRUE;
 			}
+		} else {
+			update_version_menu (product);
+			select_version (product->versions, druid_data.version);
+			select_severity (druid_data.product->bts->severities, "critical");
 		}
 		break;
 	}
@@ -831,6 +1047,14 @@ on_druid_next_clicked (GtkWidget *w, gpointer data)
 		if (application && application->component) {
 			select_component_row (application->component);
 		}
+
+		update_version_menu (product);
+		if (application)
+			select_version (product->versions, application->version);
+
+		if (druid_data.bug_type == BUG_REQUEST)
+			select_severity (druid_data.product->bts->severities, "enhancement");
+
 		break;
 	}
 	case STATE_COMPONENT:
@@ -849,21 +1073,6 @@ on_druid_next_clicked (GtkWidget *w, gpointer data)
 			gtk_widget_destroy (d);
 			return;
 		}
-		s = buddy_get_text ("the-version-entry");
-		if (!s[0] && !getenv ("BUG_ME_HARDER")) {
-			g_free (s);
-			d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
-						    0,
-						    GTK_MESSAGE_ERROR,
-						    GTK_BUTTONS_OK,
-						    _("You must specify a version for your bug report."));
-			gtk_dialog_set_default_response (GTK_DIALOG (d),
-							 GTK_RESPONSE_OK);
-			gtk_dialog_run (GTK_DIALOG (d));
-			gtk_widget_destroy (d);
-			return;
-		}
-		g_free (s);
 		if (component != druid_data.component) {
 			druid_data.component = component;
 			bugzilla_add_mostfreq (druid_data.product->bts);
